@@ -6,11 +6,7 @@ from typing import List, Optional, Type
 import numpy as np
 
 from mohou.encoder import EncoderBase, ImageEncoder, VectorIdenticalEncoder
-from mohou.encoding_rule import (
-    CompositeEncodingRule,
-    CovarianceBasedScaleBalancer,
-    EncodingRule,
-)
+from mohou.encoding_rule import CovarianceBasedScaleBalancer, EncodingRule
 from mohou.model import AutoEncoderBase
 from mohou.model.chimera import Chimera
 from mohou.propagator import Propagator, PropagatorBaseT
@@ -70,13 +66,14 @@ def load_default_image_encoder(project_path: Path) -> ImageEncoder:
 
     if tcache_autoencoder.best_model is None:
         raise DefaultNotFoundError("traincache was found but best model is not saved ")
-    return tcache_autoencoder.best_model.get_encoder()
+    return ImageEncoder.from_auto_encoder(tcache_autoencoder.best_model)
 
 
 @lru_cache(maxsize=40)
 def create_default_encoding_rule(
     project_path: Path,
     include_image_encoder: bool = True,
+    use_balancer: bool = True,
 ) -> EncodingRule:
 
     bundle = EpisodeBundle.load(project_path)
@@ -90,31 +87,44 @@ def create_default_encoding_rule(
 
     if AngleVector in bundle_spec.type_shape_table:
         av_dim = bundle_spec.type_shape_table[AngleVector][0]
-        av_idendical_encoder = VectorIdenticalEncoder(AngleVector, av_dim)
+        av_idendical_encoder = VectorIdenticalEncoder.create(AngleVector, av_dim)
         encoders.append(av_idendical_encoder)
 
     if GripperState in bundle_spec.type_shape_table:
-        gs_identital_func = VectorIdenticalEncoder(
+        gs_identital_func = VectorIdenticalEncoder.create(
             GripperState, bundle_spec.type_shape_table[GripperState][0]
         )
         encoders.append(gs_identital_func)
 
     if AnotherGripperState in bundle_spec.type_shape_table:
-        ags_identital_func = VectorIdenticalEncoder(
+        ags_identital_func = VectorIdenticalEncoder.create(
             AnotherGripperState, bundle_spec.type_shape_table[AnotherGripperState][0]
         )
         encoders.append(ags_identital_func)
 
-    tf_identical_func = VectorIdenticalEncoder(TerminateFlag, 1)
+    tf_identical_func = VectorIdenticalEncoder.create(TerminateFlag, 1)
     encoders.append(tf_identical_func)
 
     p = CovarianceBasedScaleBalancer.get_json_file_path(project_path)
     if p.exists():  # use cached balacner
-        logger.warning("warn: loading cached CovarianceBasedScaleBalancer")
-        balancer = CovarianceBasedScaleBalancer.load(project_path)
+        balancer: Optional[CovarianceBasedScaleBalancer]
+        if use_balancer:
+            logger.warning(
+                "warn: loading cached CovarianceBasedScaleBalancer. This feature is experimental."
+            )
+            balancer = CovarianceBasedScaleBalancer.load(project_path)
+        else:
+            balancer = None
         encoding_rule = EncodingRule.from_encoders(encoders, bundle=None, scale_balancer=balancer)
     else:
-        encoding_rule = EncodingRule.from_encoders(encoders, bundle=bundle, scale_balancer=None)
+        bundle_for_balancer: Optional[EpisodeBundle]
+        if use_balancer:
+            bundle_for_balancer = bundle
+        else:
+            bundle_for_balancer = None
+        encoding_rule = EncodingRule.from_encoders(
+            encoders, bundle=bundle_for_balancer, scale_balancer=None
+        )
 
     # TODO: Move The following check to unittest? but it's diffcult becaues
     # using this function pre-require the existence of trained AE ...
@@ -156,16 +166,12 @@ def create_default_chimera_propagator(project_path: Path):
     # TODO: move to inside of create_default_propagator
 
     logger.warning("warn: this feature is quite experimental. maybe deleted without notfication")
-    image_encoder = load_default_image_encoder(project_path)
 
     tcache_chimera = TrainCache.load(project_path, Chimera)
     chimera_model = tcache_chimera.best_model
 
-    image_encoder = chimera_model.ae.get_encoder()
-    image_encoding_rule = EncodingRule.from_encoders([image_encoder])
-    other_encoding_Rule = create_default_encoding_rule(project_path, include_image_encoder=False)
-
-    rule = CompositeEncodingRule([image_encoding_rule, other_encoding_Rule])
+    rule = create_default_encoding_rule(project_path)
+    rule[RGBImage] = ImageEncoder.from_auto_encoder(chimera_model.ae)
     propagator = Propagator(chimera_model.lstm, rule)
     return propagator
 
