@@ -288,6 +288,10 @@ class RGBImage(ColorImageBase["RGBImage"]):
     def resize(self, shape2d_new: Tuple[int, int]) -> None:
         self._data = cv2.resize(self._data, shape2d_new, interpolation=cv2.INTER_AREA)
 
+    def bgr2rgb(self) -> "RGBImage":
+        data_new = self._data[..., ::-1].copy()
+        return RGBImage(data_new)
+
 
 class GrayImage(ColorImageBase["GrayImage"]):
     _channel: ClassVar[int] = 1
@@ -953,6 +957,9 @@ class BundleSpec(HasTypeShapeTable):
             yaml.dump(self.to_dict(), f, default_flow_style=False, sort_keys=False)
 
 
+# If classmethod + lru_cache combination is easy, we should replace this by lru_cache
+# If you are unsatisfied by this, please make a PR using lru_cache
+# there is something like this https://gist.github.com/pavelpy/69f8cdad94aaf59abcf879bda66bfd1a
 _bundle_cache: Dict[Tuple[Path, Optional[str]], "EpisodeBundle"] = {}  # used EpisodeBundle.load
 
 
